@@ -16,7 +16,7 @@ void ToolitemDragDrop::attachMetadata(GtkWidget* w, int id, AbstractToolItem* ai
     d->id = id;
     d->item = ait;
     d->type = TOOL_ITEM_ITEM;
-    d->namedColor = nullptr;
+    d->paletteColorIndex = 0;
 
     g_object_set_data_full(G_OBJECT(w), ATTACH_DRAG_DROP_DATA, d, static_cast<GDestroyNotify>(g_free));
 }
@@ -27,7 +27,7 @@ auto ToolitemDragDrop::ToolItemDragDropData_new(AbstractToolItem* item) -> std::
     d->id = -1;
     d->item = item;
     d->type = TOOL_ITEM_ITEM;
-    d->namedColor = nullptr;
+    d->paletteColorIndex = 0;
 
     return d;
 }
@@ -38,18 +38,18 @@ void ToolitemDragDrop::attachMetadata(GtkWidget* w, int id, ToolItemType type) {
     d->id = id;
     d->item = nullptr;
     d->type = type;
-    d->namedColor = nullptr;
+    d->paletteColorIndex = 0;
 
     g_object_set_data_full(G_OBJECT(w), ATTACH_DRAG_DROP_DATA, d, static_cast<GDestroyNotify>(g_free));
 }
 
-void ToolitemDragDrop::attachMetadataColor(GtkWidget* w, int id, const NamedColor* namedColor, AbstractToolItem* item) {
+void ToolitemDragDrop::attachMetadataColor(GtkWidget* w, int id, size_t paletteColorIndex, AbstractToolItem* item) {
     ToolItemDragDropData* d = g_new(ToolItemDragDropData, 1);
     d->identify = ToolItemDragDropData_Identify;
     d->id = id;
     d->item = item;
     d->type = TOOL_ITEM_COLOR;
-    d->namedColor = namedColor;
+    d->paletteColorIndex = paletteColorIndex;
 
     g_object_set_data_full(G_OBJECT(w), ATTACH_DRAG_DROP_DATA, d, static_cast<GDestroyNotify>(g_free));
 }
@@ -69,43 +69,10 @@ auto ToolitemDragDrop::getIcon(ToolItemDragDropData* data) -> GtkWidget* {
     return gtk_image_new();
 }
 
-auto ToolitemDragDrop::getPixbuf(ToolItemDragDropData* data) -> GdkPixbuf* {
-    if (data->type == TOOL_ITEM_ITEM || data->type == TOOL_ITEM_COLOR) {
-        return data->item->getNewToolPixbuf();
-    }
-    if (data->type == TOOL_ITEM_SEPARATOR) {
-        return ToolbarSeparatorImage::getNewToolPixbuf(SeparatorType::SEPARATOR);
-    }
-    if (data->type == TOOL_ITEM_SPACER) {
-        return ToolbarSeparatorImage::getNewToolPixbuf(SeparatorType::SPACER);
-    }
-    g_error("ToolitemDragDrop::getIcon unhandled type: %i\n", data->type);
-}
-
 auto ToolitemDragDrop::checkToolItemDragDropData(ToolItemDragDropData const* d) -> bool {
     return d->identify == ToolItemDragDropData_Identify;
 }
 
-auto ToolitemDragDrop::isToolItemEnabled(ToolItemDragDropData* d) -> bool {
-    if (!checkToolItemDragDropData(d)) {
-        g_warning("ToolitemDragDrop::isToolItemEnabled data is not an instance of ToolItemDragDropData!");
-        return false;
-    }
-    if (d->type == TOOL_ITEM_SEPARATOR) {
-        return true;
-    }
-    if (d->type == TOOL_ITEM_SPACER) {
-        return true;
-    }
-    if (d->type == TOOL_ITEM_COLOR && d->item == nullptr) {
-        return true;
-    }
-
-    g_return_val_if_fail(d->item != nullptr, true);
-
-    return d->item->isEnabled();
-}
-
 auto ToolitemDragDrop::metadataGetMetadata(GtkWidget* w) -> ToolItemDragDropData* {
     void* ptr = g_object_get_data(G_OBJECT(w), ATTACH_DRAG_DROP_DATA);
     if (ptr == nullptr) {
