@@ -17,7 +17,8 @@
 #include <gdk/gdk.h>  // for GdkEvent, gdk_event_free, gdk_event_copy
 #include <glib.h>     // for gdouble, gchar, guint, guint32
 
-#include "model/Point.h"  // for Point, Point::NO_PRESSURE
+#include "model/Point.h"  // for Point::NO_PRESSURE
+#include "util/Point.h"
 
 #include "DeviceId.h"
 
@@ -35,9 +36,7 @@ enum InputEventType {
     PROXIMITY_IN_EVENT,
     PROXIMITY_OUT_EVENT,
     SCROLL_EVENT,
-    GRAB_BROKEN_EVENT,
-    KEY_PRESS_EVENT,
-    KEY_RELEASE_EVENT
+    GRAB_BROKEN_EVENT
 };
 
 enum InputDeviceClass {
@@ -45,8 +44,6 @@ enum InputDeviceClass {
     INPUT_DEVICE_PEN,
     INPUT_DEVICE_ERASER,
     INPUT_DEVICE_TOUCHSCREEN,
-    INPUT_DEVICE_KEYBOARD,
-    INPUT_DEVICE_MOUSE_KEYBOARD_COMBO,
     INPUT_DEVICE_IGNORE
 };
 
@@ -62,7 +59,7 @@ struct GdkEventGuard {
         return *this;
     }
 
-    operator GdkEvent*() const { return event.get(); }
+    GdkEvent* get() const { return event.get(); }
 
     // it's more performant to manage the GdkEvent over C++ than over gdk
     // Since the gdk_copy is extreme expansive
@@ -70,18 +67,16 @@ struct GdkEventGuard {
 };
 
 struct InputEvent final {
-    /*explicit(false)*/ explicit operator bool() const { return !!sourceEvent.event; }
+    /*explicit(false)*/ explicit operator bool() const { return device; }
 
-    GdkEventGuard sourceEvent;
+    GdkDevice* device{nullptr};  ///< Source device. Avoid using if possible.
 
     InputEventType type{UNKNOWN};
     InputDeviceClass deviceClass{INPUT_DEVICE_IGNORE};
     const gchar* deviceName{};
 
-    gdouble absoluteX{0};
-    gdouble absoluteY{0};
-    gdouble relativeX{0};
-    gdouble relativeY{0};
+    xoj::util::Point<double> absolute;  ///< In GdkSurface coordinates
+    xoj::util::Point<double> relative;  ///< In XournalWidget coordinates
 
     guint button{0};
     GdkModifierType state{};
@@ -93,11 +88,14 @@ struct InputEvent final {
     DeviceId deviceId;
 };
 
-class InputEvents {
+struct KeyEvent final {
+    guint keyval{0};
+    GdkModifierType state{};  ///< Consumed modifiers have been masked out
 
-    static InputEventType translateEventType(GdkEventType type);
+    GdkEventGuard sourceEvent;  ///< Original GdkEvent. Avoid using if possible.
+};
 
-public:
+struct InputEvents {
     static InputDeviceClass translateDeviceType(GdkDevice* device, Settings* settings);
     static InputDeviceClass translateDeviceType(const std::string& name, GdkInputSource source, Settings* settings);
 
