@@ -12,11 +12,13 @@
 #include <glib.h>     // for gdouble, gint, g_message
 #include <gtk/gtk.h>  // for gtk_adjustment_get_value
 
+#include "control/Control.h"                    // for Control
 #include "control/ToolEnums.h"                  // for TOOL_HAND, TOOL_IMAGE
 #include "control/ToolHandler.h"                // for ToolHandler
 #include "control/settings/Settings.h"          // for Settings
 #include "control/tools/CursorSelectionType.h"  // for CursorSelectionType
 #include "control/tools/EditSelection.h"        // for EditSelection
+#include "control/zoom/ZoomControl.h"           // for ZoomControl
 #include "gui/Layout.h"                         // for Layout
 #include "gui/PageView.h"                       // for XojPageView
 #include "gui/XournalView.h"                    // for XournalView
@@ -246,7 +248,8 @@ bool PenInputHandler::isCurrentTapSelection(InputEvent const& event) const {
 
     settings->getStrokeFilter(&tapMaxDuration, &tapMaxDistance, &filterRepetitionTime);
 
-    const double dpmm = settings->getDisplayDpi() / 25.4;
+    const double dpmm = inputContext->getView()->getControl()->getZoomControl()->getZoom100Value() *
+                        Util::DPI_NORMALIZATION_FACTOR / 25.4;
     const double dist = this->sequenceStartPosition.distance(event.absolute);
 
     const bool noMovement = dist < tapMaxDistance * dpmm;
