@@ -39,7 +39,6 @@ TouchDrawingInputHandler::TouchDrawingInputHandler(InputContext* inputContext):
 TouchDrawingInputHandler::~TouchDrawingInputHandler() = default;
 
 auto TouchDrawingInputHandler::handleImpl(InputEvent const& event) -> bool {
-    auto* mainWindow = inputContext->getView()->getControl()->getWindow();
     ToolHandler* toolHandler = this->inputContext->getToolHandler();
 
     // Do we need to end the touch sequence?
@@ -50,6 +49,7 @@ auto TouchDrawingInputHandler::handleImpl(InputEvent const& event) -> bool {
     // Note: Drawing outside window doesn't seem to work
     //  if this is put later in handleImpl.
     if (event.type == ENTER_EVENT) {
+        this->changeTool(event);
         this->actionEnterWindow(event);
         return false;
     }
@@ -63,11 +63,6 @@ auto TouchDrawingInputHandler::handleImpl(InputEvent const& event) -> bool {
     if ((this->primarySequence && this->primarySequence != event.sequence) || this->secondarySequence) {
         if (!this->secondarySequence) {
             this->secondarySequence = event.sequence;
-
-            // Let Gtk's touchscreen-based scrolling do
-            // momentum-based scrolling!
-            mainWindow->setGtkTouchscreenScrollingEnabled(true);
-
             XojPageView* currentPage = this->getPageAtCurrentPosition(event);
 
             if (currentPage) {
@@ -80,16 +75,12 @@ auto TouchDrawingInputHandler::handleImpl(InputEvent const& event) -> bool {
             if (event.sequence == this->primarySequence) {
                 this->primarySequence = this->secondarySequence;
                 this->secondarySequence = nullptr;
-
-                // Only scrolling now if using the hand tool.
-                mainWindow->setGtkTouchscreenScrollingEnabled(toolHandler->getToolType() == TOOL_HAND);
             } else if (event.sequence == this->secondarySequence) {
                 this->secondarySequence = nullptr;
             }
         }
 
-        // false lets another input handler (e.g. TouchInputHandler)
-        // handle the event.
+        // false lets another input handler (e.g. TouchInputHandler) handle the event.
         return false;
     }
 
@@ -100,8 +91,6 @@ auto TouchDrawingInputHandler::handleImpl(InputEvent const& event) -> bool {
 
         this->actionStart(event);
 
-        updateKineticScrollingEnabled();
-
         return false;
     }
 
@@ -112,7 +101,7 @@ auto TouchDrawingInputHandler::handleImpl(InputEvent const& event) -> bool {
     if (this->deviceClassPressed && event.type == MOTION_EVENT && toolHandler->getToolType() != TOOL_HAND) {
         GtkXournal* xournal = inputContext->getXournal();
 
-        this->inputContext->getView()->getCursor()->setRotationAngle(event.relativeX);
+        this->inputContext->getView()->getCursor()->setRotationAngle(event.relative.x);
 
         this->actionMotion(event);
         XournalppCursor* cursor = xournal->view->getCursor();
@@ -140,8 +129,9 @@ auto TouchDrawingInputHandler::changeTool(InputEvent const& event) -> bool {
     bool toolChanged = false;
 
     if (cfg->device == event.deviceName) {
-        if (InputUtils::touchDrawingDisallowed(toolHandler, settings))
+        if (InputUtils::touchDrawingDisallowed(toolHandler, settings)) {
             return false;
+        }
         toolChanged = InputUtils::applyButton(toolHandler, settings, Button::BUTTON_TOUCH);
     } else {
         toolChanged = toolHandler->pointActiveToolToToolbarTool();
@@ -161,8 +151,7 @@ void TouchDrawingInputHandler::updateKineticScrollingEnabled() {
     auto* mainWindow = control->getWindow();
     auto* toolHandler = this->inputContext->getToolHandler();
 
-    //  Kinetic scrolling is nice; however, we need to disable it so we can draw (it steals
-    // single-finger input).
+    // Kinetic scrolling is nice; however, we need to disable it so we can draw (it steals single-finger input).
     if (mainWindow != nullptr && control->getSettings()->getTouchDrawingEnabled()) {
         mainWindow->setGtkTouchscreenScrollingEnabled(toolHandler->getToolType() == TOOL_HAND);
     }
