@@ -7,22 +7,28 @@
 
 #include "control/Control.h"   // for Control
 #include "control/PdfCache.h"  // for PdfCache
+#include "gui/Builder.h"       // for Builder
 #include "gui/MainWindow.h"    // for MainWindow
 #include "model/Document.h"    // for Document
 #include "util/Util.h"         // for npos
 #include "util/glib_casts.h"   // for wrap_for_once_v
+#include "util/gtk4_helper.h"
 
 #include "SidebarLayout.h"            // for SidebarLayout
 #include "SidebarPreviewBaseEntry.h"  // for SidebarPreviewBaseEntry
 
-class GladeGui;
+constexpr auto XML_FILE = "sidebar.ui";
 
-
-SidebarPreviewBase::SidebarPreviewBase(Control* control, GladeGui* gui, SidebarToolbar* toolbar):
-        AbstractSidebarPage(control, toolbar),
-        scrollPreview(gtk_scrolled_window_new(nullptr, nullptr), xoj::util::adopt),
-        iconViewPreview(gtk_layout_new(nullptr, nullptr), xoj::util::adopt) {
-    this->layoutmanager = new SidebarLayout();
+SidebarPreviewBase::SidebarPreviewBase(Control* control, const char* menuId, const char* toolbarId):
+        AbstractSidebarPage(control),
+        scrollableBox(gtk_scrolled_window_new(), xoj::util::adopt),
+        mainBox(gtk_box_new(GTK_ORIENTATION_VERTICAL, 0), xoj::util::adopt),
+        miniaturesContainer(GTK_FIXED(gtk_fixed_new()), xoj::util::adopt) {
+    gtk_box_append(GTK_BOX(mainBox.get()), scrollableBox.get());
+    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollableBox.get()), GTK_POLICY_AUTOMATIC,
+                                   GTK_POLICY_AUTOMATIC);
+    gtk_scrolled_window_set_child(GTK_SCROLLED_WINDOW(scrollableBox.get()), GTK_WIDGET(miniaturesContainer.get()));
+    gtk_widget_set_vexpand(scrollableBox.get(), true);
 
     Document* doc = this->control->getDocument();
     doc->lock();
@@ -31,43 +37,44 @@ SidebarPreviewBase::SidebarPreviewBase(Control* control, GladeGui* gui, SidebarT
     }
     doc->unlock();
 
-    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(this->scrollPreview.get()), GTK_POLICY_AUTOMATIC,
-                                   GTK_POLICY_AUTOMATIC);
-    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(this->scrollPreview.get()), GTK_SHADOW_IN);
-
-    gtk_container_add(GTK_CONTAINER(this->scrollPreview.get()), this->iconViewPreview.get());
 
     registerListener(this->control);
     this->control->addChangedDocumentListener(this);
 
-    g_signal_connect(this->scrollPreview.get(), "size-allocate", G_CALLBACK(sizeChanged), this);
+    auto* adj = gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(scrollableBox.get()));
+    g_signal_connect(
+            adj, "notify::page-size", G_CALLBACK(+[](GObject* adj, GParamSpec*, gpointer d) {
+                static_cast<SidebarPreviewBase*>(d)->newWidth(gtk_adjustment_get_page_size(GTK_ADJUSTMENT(adj)));
+            }),
+            this);
 
-    gtk_widget_show_all(this->scrollPreview.get());
+    Builder builder(control->getGladeSearchPath(), XML_FILE);
+    GMenuModel* menu = G_MENU_MODEL(builder.get<GObject>(menuId));
+    contextMenu.reset(GTK_MENU(gtk_menu_new_from_model(menu)), xoj::util::adopt);
+    gtk_menu_attach_to_widget(contextMenu.get(), mainBox.get(), nullptr);
+
+    gtk_box_append(GTK_BOX(mainBox.get()), builder.get(toolbarId));
+
+    gtk_widget_show_all(mainBox.get());
 }
 
-SidebarPreviewBase::~SidebarPreviewBase() {
-    delete this->layoutmanager;
-    this->layoutmanager = nullptr;
+SidebarPreviewBase::~SidebarPreviewBase() { this->control->removeChangedDocumentListener(this); }
 
-    this->control->removeChangedDocumentListener(this);
-
-    this->previews.clear();
+void SidebarPreviewBase::enableSidebar() {
+    if (!this->enabled) {
+        enabled = true;
+        layout();
+    }
 }
 
-void SidebarPreviewBase::enableSidebar() { enabled = true; }
-
 void SidebarPreviewBase::disableSidebar() { enabled = false; }
 
-void SidebarPreviewBase::sizeChanged(GtkWidget* widget, GtkAllocation* allocation, SidebarPreviewBase* sidebar) {
-    static int lastWidth = -1;
+void SidebarPreviewBase::newWidth(double width) {
+    static constexpr double TRIGGER = 20.;
 
-    if (lastWidth == -1) {
-        lastWidth = allocation->width;
-    }
-
-    if (std::abs(lastWidth - allocation->width) > 20) {
-        sidebar->layout();
-        lastWidth = allocation->width;
+    if (std::abs(lastWidth - width) > TRIGGER) {
+        this->layout();
+        lastWidth = width;
     }
 }
 
@@ -75,11 +82,15 @@ auto SidebarPreviewBase::getZoom() const -> double { return this->zoom; }
 
 auto SidebarPreviewBase::getCache() -> PdfCache* { return this->cache.get(); }
 
-void SidebarPreviewBase::layout() { SidebarLayout::layout(this); }
+void SidebarPreviewBase::layout() {
+    if (enabled) {
+        SidebarLayout::layout(this);
+    }
+}
 
 auto SidebarPreviewBase::hasData() -> bool { return true; }
 
-auto SidebarPreviewBase::getWidget() -> GtkWidget* { return this->scrollPreview.get(); }
+auto SidebarPreviewBase::getWidget() -> GtkWidget* { return this->mainBox.get(); }
 
 void SidebarPreviewBase::documentChanged(DocumentChangeType type) {
     if (type == DOCUMENT_CHANGE_COMPLETE || type == DOCUMENT_CHANGE_CLEARED) {
@@ -114,8 +125,7 @@ auto SidebarPreviewBase::scrollToPreview(SidebarPreviewBase* sidebar) -> bool {
         auto& p = sidebar->previews[sidebar->selectedEntry];
 
         // scroll to preview
-        GtkAdjustment* hadj = gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(sidebar->scrollPreview.get()));
-        GtkAdjustment* vadj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(sidebar->scrollPreview.get()));
+        GtkAdjustment* vadj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(sidebar->scrollableBox.get()));
         GtkWidget* widget = p->getWidget();
 
         GtkAllocation allocation;
@@ -129,7 +139,6 @@ auto SidebarPreviewBase::scrollToPreview(SidebarPreviewBase* sidebar) -> bool {
         }
 
         gtk_adjustment_clamp_page(vadj, y, y + allocation.height);
-        gtk_adjustment_clamp_page(hadj, x, x + allocation.width);
     }
     return false;
 }
@@ -137,3 +146,7 @@ auto SidebarPreviewBase::scrollToPreview(SidebarPreviewBase* sidebar) -> bool {
 void SidebarPreviewBase::pageDeleted(size_t page) {}
 
 void SidebarPreviewBase::pageInserted(size_t page) {}
+
+void SidebarPreviewBase::openPreviewContextMenu(GdkEvent* currentEvent) {
+    gtk_menu_popup_at_pointer(contextMenu.get(), currentEvent);
+}
