@@ -1,11 +1,11 @@
-#include <cassert>  // for assert
-#include <memory>   // for make_unique, unique_ptr
+#include <memory>  // for make_unique, unique_ptr
 
 #include "model/Element.h"   // for Element, ELEMENT_IMAGE, ELEMENT_STROKE
 #include "model/Image.h"     // for Image
 #include "model/Stroke.h"    // for Stroke
 #include "model/TexImage.h"  // for TexImage
 #include "model/Text.h"      // for Text
+#include "util/Assert.h"     // for xoj_assert
 
 #include "ImageView.h"     // for ImageView
 #include "StrokeView.h"    // for StrokeView
@@ -26,7 +26,7 @@ auto ElementView::createFromElement(const Element* e) -> std::unique_ptr<Element
         case ELEMENT_TEXIMAGE:
             return std::make_unique<TexImageView>(dynamic_cast<const TexImage*>(e));
         default:
-            assert(false && "ElementView::getFromElement: Unknown element type!");
+            xoj_assert_message(false, "ElementView::getFromElement: Unknown element type!");
             return nullptr;
     }
 }
