@@ -32,8 +32,12 @@ struct GSourceURef {
         }
     }
 
+    operator bool() const { return id != 0; }
+
     /// When the GSource is run
     void consume() { id = 0; }
+    /// To cancel the callback
+    void cancel() { *this = 0; }
 
 private:
     unsigned int id = 0;  // handler id
