@@ -12,18 +12,22 @@
 #include <cmath>
 #include <filesystem>
 #include <iostream>
+#include <string_view>
 
 #include <config-test.h>
 #include <gtest/gtest.h>
 
 #include "control/xojfile/LoadHandler.h"
 #include "control/xojfile/SaveHandler.h"
+#include "model/Element.h"
 #include "model/Image.h"
+#include "model/PageRef.h"
 #include "model/Stroke.h"
 #include "model/TexImage.h"
 #include "model/Text.h"
 #include "model/XojPage.h"
 #include "util/PathUtil.h"
+#include "util/StringUtils.h"
 
 #include "filesystem.h"
 
@@ -39,60 +43,60 @@ using std::string;
 void testLoadStoreLoadHelper(const fs::path& filepath, double tol = 1e-8) {
     auto getElements = [](Document* doc) {
         EXPECT_EQ((size_t)1, doc->getPageCount());
-        PageRef page = doc->getPage(0);
+        ConstPageRef page = doc->getPage(0);
 
-        EXPECT_EQ((size_t)1, (*page).getLayerCount());
-        Layer* layer = (*(*page).getLayers())[0];
+        EXPECT_EQ((size_t)1, page->getLayerCount());
+        const Layer* layer = page->getLayersView()[0];
 
-        const std::vector<Element*>& elements = layer->getElements();
-        EXPECT_EQ(8, static_cast<int>(layer->getElements().size()));
+        auto elements = layer->getElementsView();
+        EXPECT_EQ(8, static_cast<int>(elements.size()));
 
-        Stroke* e0 = (Stroke*)layer->getElements()[0];
+        Stroke* e0 = (Stroke*)elements[0];
         EXPECT_EQ(ELEMENT_STROKE, e0->getType());
 
-        Stroke* e1 = (Stroke*)layer->getElements()[1];
+        Stroke* e1 = (Stroke*)elements[1];
         EXPECT_EQ(ELEMENT_STROKE, e1->getType());
 
-        Stroke* e2 = (Stroke*)layer->getElements()[2];
+        Stroke* e2 = (Stroke*)elements[2];
         EXPECT_EQ(ELEMENT_STROKE, e2->getType());
 
-        Stroke* e3 = (Stroke*)layer->getElements()[3];
+        Stroke* e3 = (Stroke*)elements[3];
         EXPECT_EQ(ELEMENT_STROKE, e3->getType());
 
-        Stroke* e4 = (Stroke*)layer->getElements()[4];
+        Stroke* e4 = (Stroke*)elements[4];
         EXPECT_EQ(ELEMENT_STROKE, e4->getType());
 
-        Text* e5 = (Text*)layer->getElements()[5];
+        Text* e5 = (Text*)elements[5];
         EXPECT_EQ(ELEMENT_TEXT, e5->getType());
 
-        Stroke* e6 = (Stroke*)layer->getElements()[6];
+        Stroke* e6 = (Stroke*)elements[6];
         EXPECT_EQ(ELEMENT_STROKE, e6->getType());
 
-        Stroke* e7 = (Stroke*)layer->getElements()[7];
+        Stroke* e7 = (Stroke*)elements[7];
         EXPECT_EQ(ELEMENT_STROKE, e7->getType());
 
         return elements;
     };
     LoadHandler handler;
-    Document* doc1 = handler.loadDocument(filepath);
-    auto elements1 = getElements(doc1);
+    auto doc1 = handler.loadDocument(filepath);
+    auto elements1 = getElements(doc1.get());
 
     SaveHandler h;
-    h.prepareSave(doc1);
     auto tmp = Util::getTmpDirSubfolder() / "save.xopp";
+    h.prepareSave(doc1.get(), tmp);
     h.saveTo(tmp);
 
     // Create a second loader so the first one doesn't free the memory
     LoadHandler handler2;
-    Document* doc2 = handler2.loadDocument(tmp);
-    auto elements2 = getElements(doc2);
+    auto doc2 = handler2.loadDocument(tmp);
+    auto elements2 = getElements(doc2.get());
 
     // Check that the coordinates from both files don't differ more than the precision they were saved with
     auto coordEq = [tol](double a, double b) { return std::abs(a - b) <= tol; };
 
     for (unsigned long i = 0; i < elements1.size(); i++) {
-        Element* a = elements1.at(i);
-        Element* b = elements2.at(i);
+        const Element* a = elements1[i];
+        const Element* b = elements2[i];
         EXPECT_EQ(a->getType(), b->getType());
         EXPECT_TRUE(coordEq(a->getX(), b->getX()));
         EXPECT_TRUE(coordEq(a->getY(), b->getY()));
@@ -101,13 +105,13 @@ void testLoadStoreLoadHelper(const fs::path& filepath, double tol = 1e-8) {
         EXPECT_EQ(a->getColor(), b->getColor());
         switch (a->getType()) {
             case ELEMENT_STROKE: {
-                auto sA = dynamic_cast<Stroke*>(a);
-                auto sB = dynamic_cast<Stroke*>(b);
+                auto sA = dynamic_cast<const Stroke*>(a);
+                auto sB = dynamic_cast<const Stroke*>(b);
                 EXPECT_EQ(sA->getPointCount(), sB->getPointCount());
                 EXPECT_EQ(sA->getToolType(), sB->getToolType());
                 EXPECT_EQ(sA->getLineStyle().hasDashes(), sB->getLineStyle().hasDashes());
                 EXPECT_TRUE(coordEq(sA->getAvgPressure(), sB->getAvgPressure()));
-                for (int j = 0; j < sA->getPointCount(); j++) {
+                for (size_t j = 0; j < sA->getPointCount(); j++) {
                     Point pA = sA->getPoint(j);
                     Point pB = sB->getPoint(j);
                     EXPECT_TRUE(coordEq(pA.x, pB.x));
@@ -117,8 +121,8 @@ void testLoadStoreLoadHelper(const fs::path& filepath, double tol = 1e-8) {
                 break;
             }
             case ELEMENT_TEXT: {
-                auto tA = dynamic_cast<Text*>(a);
-                auto tB = dynamic_cast<Text*>(b);
+                auto tA = dynamic_cast<const Text*>(a);
+                auto tB = dynamic_cast<const Text*>(b);
                 EXPECT_EQ(tA->getText(), tB->getText());
                 EXPECT_EQ(tA->getFontSize(), tB->getFontSize());
                 break;
@@ -131,98 +135,103 @@ void testLoadStoreLoadHelper(const fs::path& filepath, double tol = 1e-8) {
     }
 }
 
-void checkPageType(Document* doc, int pageIndex, string expectedText, PageType expectedBgType) {
-    PageRef page = doc->getPage(pageIndex);
+void checkPageType(const Document* doc, size_t pageIndex, string expectedText, PageType expectedBgType) {
+    ConstPageRef page = doc->getPage(pageIndex);
 
     PageType bgType = page->getBackgroundType();
     EXPECT_TRUE(expectedBgType == bgType);
 
-    EXPECT_EQ((size_t)1, (*page).getLayerCount());
-    Layer* layer = (*(*page).getLayers())[0];
+    EXPECT_EQ((size_t)1, page->getLayerCount());
+    const Layer* layer = page->getLayersView()[0];
 
-    Element* element = layer->getElements().front();
+    auto* element = layer->getElementsView().front();
     EXPECT_EQ(ELEMENT_TEXT, element->getType());
 
-    Text* text = (Text*)element;
+    auto* text = dynamic_cast<const Text*>(element);
+    EXPECT_NE(text, nullptr);
     EXPECT_EQ(expectedText, text->getText());
 }
 
 
-void checkLayer(PageRef page, int layerIndex, string expectedText) {
-    Layer* layer = (*(*page).getLayers())[layerIndex];
+void checkLayer(ConstPageRef page, size_t layerIndex, string expectedText) {
+    const Layer* layer = page->getLayersView()[layerIndex];
 
-    Element* element = layer->getElements().front();
+    auto* element = layer->getElementsView().front();
 
     EXPECT_EQ(ELEMENT_TEXT, element->getType());
 
-    Text* text = (Text*)element;
+    auto* text = dynamic_cast<const Text*>(element);
+    EXPECT_NE(text, nullptr);
     EXPECT_EQ(expectedText, text->getText());
 }
 
 TEST(ControlLoadHandler, testLoad) {
     LoadHandler handler;
-    Document* doc = handler.loadDocument(GET_TESTFILE("test1.xoj"));
+    auto doc = handler.loadDocument(GET_TESTFILE(u8"test1.xoj"));
 
     EXPECT_EQ((size_t)1, doc->getPageCount());
-    PageRef page = doc->getPage(0);
+    ConstPageRef page = doc->getPage(0);
 
-    EXPECT_EQ((size_t)1, (*page).getLayerCount());
-    Layer* layer = (*(*page).getLayers())[0];
+    EXPECT_EQ((size_t)1, page->getLayerCount());
+    const Layer* layer = page->getLayersView()[0];
 
-    Element* element = layer->getElements().front();
+    auto* element = layer->getElementsView().front();
     EXPECT_EQ(ELEMENT_TEXT, element->getType());
 
-    Text* text = (Text*)element;
+    auto* text = dynamic_cast<const Text*>(element);
+    EXPECT_NE(text, nullptr);
 
     EXPECT_EQ(string("12345"), text->getText());
 }
 
 TEST(ControlLoadHandler, testLoadZipped) {
     LoadHandler handler;
-    Document* doc = handler.loadDocument(GET_TESTFILE("packaged_xopp/test.xopp"));
+    auto doc = handler.loadDocument(GET_TESTFILE(u8"packaged_xopp/test.xopp"));
 
     EXPECT_EQ((size_t)1, doc->getPageCount());
-    PageRef page = doc->getPage(0);
+    ConstPageRef page = doc->getPage(0);
 
-    EXPECT_EQ((size_t)1, (*page).getLayerCount());
-    Layer* layer = (*(*page).getLayers())[0];
+    EXPECT_EQ((size_t)1, page->getLayerCount());
+    const Layer* layer = page->getLayersView()[0];
 
-    Element* element = layer->getElements().front();
+    auto* element = layer->getElementsView().front();
     EXPECT_EQ(ELEMENT_TEXT, element->getType());
 
-    Text* text = (Text*)element;
+    auto* text = dynamic_cast<const Text*>(element);
+    EXPECT_NE(text, nullptr);
 
     EXPECT_EQ(string("12345"), text->getText());
 }
 
 TEST(ControlLoadHandler, testLoadUnzipped) {
     LoadHandler handler;
-    Document* doc = handler.loadDocument(GET_TESTFILE("test1.unzipped.xoj"));
+    auto doc = handler.loadDocument(GET_TESTFILE(u8"test1.unzipped.xoj"));
 
     EXPECT_EQ((size_t)1, doc->getPageCount());
-    PageRef page = doc->getPage(0);
+    ConstPageRef page = doc->getPage(0);
 
-    EXPECT_EQ((size_t)1, (*page).getLayerCount());
-    Layer* layer = (*(*page).getLayers())[0];
+    EXPECT_EQ((size_t)1, page->getLayerCount());
+    const Layer* layer = page->getLayersView()[0];
 
-    Element* element = layer->getElements().front();
+    auto* element = layer->getElementsView().front();
     EXPECT_EQ(ELEMENT_TEXT, element->getType());
 
-    Text* text = (Text*)element;
+    auto* text = dynamic_cast<const Text*>(element);
+    EXPECT_NE(text, nullptr);
 
     EXPECT_EQ(string("12345"), text->getText());
 }
 
 TEST(ControlLoadHandler, testPages) {
     LoadHandler handler;
-    Document* doc = handler.loadDocument(GET_TESTFILE("load/pages.xoj"));
+    auto doc = handler.loadDocument(GET_TESTFILE(u8"load/pages.xoj"));
 
     EXPECT_EQ((size_t)6, doc->getPageCount());
 }
 
 TEST(ControlLoadHandler, testPagesZipped) {
     LoadHandler handler;
-    Document* doc = handler.loadDocument(GET_TESTFILE("packaged_xopp/pages.xopp"));
+    auto doc = handler.loadDocument(GET_TESTFILE(u8"packaged_xopp/pages.xopp"));
 
     EXPECT_EQ((size_t)6, doc->getPageCount());
 }
@@ -230,48 +239,48 @@ TEST(ControlLoadHandler, testPagesZipped) {
 
 TEST(ControlLoadHandler, testPageType) {
     LoadHandler handler;
-    Document* doc = handler.loadDocument(GET_TESTFILE("load/pages.xoj"));
+    auto doc = handler.loadDocument(GET_TESTFILE(u8"load/pages.xoj"));
 
     EXPECT_EQ((size_t)6, doc->getPageCount());
-    checkPageType(doc, 0, "p1", PageType(PageTypeFormat::Plain));
-    checkPageType(doc, 1, "p2", PageType(PageTypeFormat::Ruled));
-    checkPageType(doc, 2, "p3", PageType(PageTypeFormat::Lined));
-    checkPageType(doc, 3, "p4", PageType(PageTypeFormat::Staves));
-    checkPageType(doc, 4, "p5", PageType(PageTypeFormat::Graph));
-    checkPageType(doc, 5, "p6", PageType(PageTypeFormat::Image));
+    checkPageType(doc.get(), 0, "p1", PageType(PageTypeFormat::Plain));
+    checkPageType(doc.get(), 1, "p2", PageType(PageTypeFormat::Ruled));
+    checkPageType(doc.get(), 2, "p3", PageType(PageTypeFormat::Lined));
+    checkPageType(doc.get(), 3, "p4", PageType(PageTypeFormat::Staves));
+    checkPageType(doc.get(), 4, "p5", PageType(PageTypeFormat::Graph));
+    checkPageType(doc.get(), 5, "p6", PageType(PageTypeFormat::Image));
 }
 
 TEST(ControlLoadHandler, testPageTypeZipped) {
     LoadHandler handler;
-    Document* doc = handler.loadDocument(GET_TESTFILE("packaged_xopp/pages.xopp"));
+    auto doc = handler.loadDocument(GET_TESTFILE(u8"packaged_xopp/pages.xopp"));
 
     EXPECT_EQ((size_t)6, doc->getPageCount());
-    checkPageType(doc, 0, "p1", PageType(PageTypeFormat::Plain));
-    checkPageType(doc, 1, "p2", PageType(PageTypeFormat::Ruled));
-    checkPageType(doc, 2, "p3", PageType(PageTypeFormat::Lined));
-    checkPageType(doc, 3, "p4", PageType(PageTypeFormat::Staves));
-    checkPageType(doc, 4, "p5", PageType(PageTypeFormat::Graph));
-    checkPageType(doc, 5, "p6", PageType(PageTypeFormat::Image));
+    checkPageType(doc.get(), 0, "p1", PageType(PageTypeFormat::Plain));
+    checkPageType(doc.get(), 1, "p2", PageType(PageTypeFormat::Ruled));
+    checkPageType(doc.get(), 2, "p3", PageType(PageTypeFormat::Lined));
+    checkPageType(doc.get(), 3, "p4", PageType(PageTypeFormat::Staves));
+    checkPageType(doc.get(), 4, "p5", PageType(PageTypeFormat::Graph));
+    checkPageType(doc.get(), 5, "p6", PageType(PageTypeFormat::Image));
 }
 
 TEST(ControlLoadHandler, testPageTypeFormatCopyFix) {
     LoadHandler handler;
-    Document* doc = handler.loadDocument(GET_TESTFILE("pageTypeFormatCopy.xopp"));
+    auto doc = handler.loadDocument(GET_TESTFILE(u8"pageTypeFormatCopy.xopp"));
 
     EXPECT_EQ((size_t)3, doc->getPageCount());
-    checkPageType(doc, 0, "p1", PageType(PageTypeFormat::Lined));
-    checkPageType(doc, 1, "p2", PageType(PageTypeFormat::Plain));  // PageTypeFormat::Copy in the file
-    checkPageType(doc, 2, "p3", PageType(PageTypeFormat::Plain));  // PageTypeFormat::Plain in the file
+    checkPageType(doc.get(), 0, "p1", PageType(PageTypeFormat::Lined));
+    checkPageType(doc.get(), 1, "p2", PageType(PageTypeFormat::Plain));  // PageTypeFormat::Copy in the file
+    checkPageType(doc.get(), 2, "p3", PageType(PageTypeFormat::Plain));  // PageTypeFormat::Plain in the file
 }
 
 TEST(ControlLoadHandler, testLayer) {
     LoadHandler handler;
-    Document* doc = handler.loadDocument(GET_TESTFILE("load/layer.xoj"));
+    auto doc = handler.loadDocument(GET_TESTFILE(u8"load/layer.xoj"));
 
     EXPECT_EQ((size_t)1, doc->getPageCount());
-    PageRef page = doc->getPage(0);
+    ConstPageRef page = doc->getPage(0);
 
-    EXPECT_EQ((size_t)3, (*page).getLayerCount());
+    EXPECT_EQ((size_t)3, page->getLayerCount());
     checkLayer(page, 0, "l1");
     checkLayer(page, 1, "l2");
     checkLayer(page, 2, "l3");
@@ -279,12 +288,12 @@ TEST(ControlLoadHandler, testLayer) {
 
 TEST(ControlLoadHandler, testLayerZipped) {
     LoadHandler handler;
-    Document* doc = handler.loadDocument(GET_TESTFILE("packaged_xopp/layer.xopp"));
+    auto doc = handler.loadDocument(GET_TESTFILE(u8"packaged_xopp/layer.xopp"));
 
     EXPECT_EQ((size_t)1, doc->getPageCount());
-    PageRef page = doc->getPage(0);
+    ConstPageRef page = doc->getPage(0);
 
-    EXPECT_EQ((size_t)3, (*page).getLayerCount());
+    EXPECT_EQ((size_t)3, page->getLayerCount());
     checkLayer(page, 0, "l1");
     checkLayer(page, 1, "l2");
     checkLayer(page, 2, "l3");
@@ -292,21 +301,24 @@ TEST(ControlLoadHandler, testLayerZipped) {
 
 TEST(ControlLoadHandler, testText) {
     LoadHandler handler;
-    Document* doc = handler.loadDocument(GET_TESTFILE("load/text.xml"));
+    auto doc = handler.loadDocument(GET_TESTFILE(u8"load/text.xml"));
 
     EXPECT_EQ((size_t)1, doc->getPageCount());
-    PageRef page = doc->getPage(0);
+    ConstPageRef page = doc->getPage(0);
 
-    EXPECT_EQ((size_t)1, (*page).getLayerCount());
-    Layer* layer = (*(*page).getLayers())[0];
+    EXPECT_EQ((size_t)1, page->getLayerCount());
+    const Layer* layer = page->getLayersView()[0];
 
-    Text* t1 = (Text*)layer->getElements()[0];
+    auto* t1 = dynamic_cast<const Text*>(layer->getElementsView()[0]);
+    EXPECT_NE(t1, nullptr);
     EXPECT_EQ(ELEMENT_TEXT, t1->getType());
 
-    Text* t2 = (Text*)layer->getElements()[1];
+    auto* t2 = dynamic_cast<const Text*>(layer->getElementsView()[1]);
+    EXPECT_NE(t2, nullptr);
     EXPECT_EQ(ELEMENT_TEXT, t2->getType());
 
-    Text* t3 = (Text*)layer->getElements()[2];
+    auto* t3 = dynamic_cast<const Text*>(layer->getElementsView()[2]);
+    EXPECT_NE(t3, nullptr);
     EXPECT_EQ(ELEMENT_TEXT, t3->getType());
 
     EXPECT_EQ(string("red"), t1->getText());
@@ -320,21 +332,24 @@ TEST(ControlLoadHandler, testText) {
 
 TEST(ControlLoadHandler, testTextZipped) {
     LoadHandler handler;
-    Document* doc = handler.loadDocument(GET_TESTFILE("packaged_xopp/text.xopp"));
+    auto doc = handler.loadDocument(GET_TESTFILE(u8"packaged_xopp/text.xopp"));
 
     EXPECT_EQ((size_t)1, doc->getPageCount());
-    PageRef page = doc->getPage(0);
+    ConstPageRef page = doc->getPage(0);
 
-    EXPECT_EQ((size_t)1, (*page).getLayerCount());
-    Layer* layer = (*(*page).getLayers())[0];
+    EXPECT_EQ((size_t)1, page->getLayerCount());
+    const Layer* layer = page->getLayersView().front();
 
-    Text* t1 = (Text*)layer->getElements()[0];
+    auto* t1 = dynamic_cast<const Text*>(layer->getElementsView()[0]);
+    EXPECT_NE(t1, nullptr);
     EXPECT_EQ(ELEMENT_TEXT, t1->getType());
 
-    Text* t2 = (Text*)layer->getElements()[1];
+    auto* t2 = dynamic_cast<const Text*>(layer->getElementsView()[1]);
+    EXPECT_NE(t2, nullptr);
     EXPECT_EQ(ELEMENT_TEXT, t2->getType());
 
-    Text* t3 = (Text*)layer->getElements()[2];
+    auto* t3 = dynamic_cast<const Text*>(layer->getElementsView()[2]);
+    EXPECT_NE(t3, nullptr);
     EXPECT_EQ(ELEMENT_TEXT, t3->getType());
 
     EXPECT_EQ(string("red"), t1->getText());
@@ -348,20 +363,20 @@ TEST(ControlLoadHandler, testTextZipped) {
 
 TEST(ControlLoadHandler, testImageZipped) {
     LoadHandler handler;
-    Document* doc = handler.loadDocument(GET_TESTFILE("packaged_xopp/imgAttachment/new.xopp"));
+    auto doc = handler.loadDocument(GET_TESTFILE(u8"packaged_xopp/imgAttachment/new.xopp"));
 
     EXPECT_EQ(1U, doc->getPageCount());
-    PageRef page = doc->getPage(0);
+    ConstPageRef page = doc->getPage(0);
     EXPECT_EQ(1U, page->getLayerCount());
-    Layer* layer = (*page->getLayers())[0];
-    EXPECT_EQ(layer->getElements().size(), 1);
+    const Layer* layer = page->getLayersView()[0];
+    EXPECT_EQ(layer->getElementsView().size(), 1);
 
-    Image* img = dynamic_cast<Image*>(layer->getElements()[0]);
+    const Image* img = dynamic_cast<const Image*>(layer->getElementsView()[0]);
     EXPECT_TRUE(img);
 }
 
 namespace {
-void checkImageFormat(Image* img, const char* formatName) {
+void checkImageFormat(const Image* img, const char* formatName) {
     GdkPixbufLoader* imgLoader = gdk_pixbuf_loader_new();
     ASSERT_TRUE(gdk_pixbuf_loader_write(imgLoader, img->getRawData(), img->getRawDataLength(), nullptr));
     ASSERT_TRUE(gdk_pixbuf_loader_close(imgLoader, nullptr));
@@ -377,15 +392,15 @@ void checkImageFormat(Image* img, const char* formatName) {
 TEST(ControlLoadHandler, imageLoadJpeg) {
     // check loading of arbitrary image format (up to whatever is supported by GdkPixbuf)
     LoadHandler handler;
-    Document* doc = handler.loadDocument(GET_TESTFILE("packaged_xopp/imgAttachment/doc_with_jpg.xopp"));
+    auto doc = handler.loadDocument(GET_TESTFILE(u8"packaged_xopp/imgAttachment/doc_with_jpg.xopp"));
     ASSERT_TRUE(doc) << "doc should not be null";
     ASSERT_EQ(1U, doc->getPageCount());
-    PageRef page = doc->getPage(0);
+    ConstPageRef page = doc->getPage(0);
     ASSERT_EQ(1U, page->getLayerCount());
-    Layer* layer = (*page->getLayers())[0];
-    ASSERT_EQ(layer->getElements().size(), 1);
+    const Layer* layer = page->getLayersView()[0];
+    ASSERT_EQ(layer->getElementsView().size(), 1);
 
-    Image* img = dynamic_cast<Image*>(layer->getElements()[0]);
+    const Image* img = dynamic_cast<const Image*>(layer->getElementsView()[0]);
     ASSERT_TRUE(img) << "element should be an image";
 
     checkImageFormat(img, "jpeg");
@@ -402,25 +417,25 @@ TEST(ControlLoadHandler, imageSaveJpegBackwardCompat) {
     // save journal containing JPEG image
     {
         LoadHandler handler;
-        Document* doc = handler.loadDocument(GET_TESTFILE("packaged_xopp/imgAttachment/doc_with_jpg.xopp"));
+        auto doc = handler.loadDocument(GET_TESTFILE(u8"packaged_xopp/imgAttachment/doc_with_jpg.xopp"));
         ASSERT_TRUE(doc) << "doc with jpeg should not be null";
 
         SaveHandler saver;
-        saver.prepareSave(doc);
+        saver.prepareSave(doc.get(), outPath);
         saver.saveTo(outPath);
     }
 
     // check that the image is saved as PNG
     LoadHandler handler;
-    Document* doc = handler.loadDocument(outPath);
+    auto doc = handler.loadDocument(outPath);
     ASSERT_TRUE(doc) << "saved doc should not be null";
     ASSERT_EQ(1U, doc->getPageCount());
-    PageRef page = doc->getPage(0);
+    ConstPageRef page = doc->getPage(0);
     ASSERT_EQ(1U, page->getLayerCount());
-    Layer* layer = (*page->getLayers())[0];
-    ASSERT_EQ(layer->getElements().size(), 1);
+    const Layer* layer = page->getLayersView()[0];
+    ASSERT_EQ(layer->getElementsView().size(), 1);
 
-    Image* img = dynamic_cast<Image*>(layer->getElements()[0]);
+    const Image* img = dynamic_cast<const Image*>(layer->getElementsView()[0]);
     ASSERT_TRUE(img) << "element should be an image";
     checkImageFormat(img, "png");
 }
@@ -433,11 +448,11 @@ TEST(ControlLoadHandler, linebreaksLatex) {
     // save journal containing latex object with linebreaks.
     {
         LoadHandler handler;
-        auto doc = handler.loadDocument(GET_TESTFILE("load/linebreaksLatex.xopp"));
+        auto doc = handler.loadDocument(GET_TESTFILE(u8"load/linebreaksLatex.xopp"));
         ASSERT_TRUE(doc) << "latex objects with linebreaks";
 
         SaveHandler saver;
-        saver.prepareSave(doc);
+        saver.prepareSave(doc.get(), outPath);
         saver.saveTo(outPath);
     }
 
@@ -448,49 +463,52 @@ TEST(ControlLoadHandler, linebreaksLatex) {
     ASSERT_EQ(1U, doc->getPageCount());
     PageRef page = doc->getPage(0);
     ASSERT_EQ(1U, page->getLayerCount());
-    Layer* layer = (*page->getLayers())[0];
+    Layer* layer = page->getLayers()[0];
 
-    TexImage* teximage = (TexImage*)layer->getElements()[0];
+    auto* teximage = static_cast<const TexImage*>(layer->getElements()[0].get());
     EXPECT_EQ("{.\n}", teximage->getText());
 
-    teximage = (TexImage*)layer->getElements()[1];
+    teximage = static_cast<const TexImage*>(layer->getElements()[1].get());
     EXPECT_EQ("{.\r}", teximage->getText());
 
-    teximage = (TexImage*)layer->getElements()[2];
+    teximage = static_cast<const TexImage*>(layer->getElements()[2].get());
     EXPECT_EQ("{.\r\n}", teximage->getText());
+
+    fs::remove(outPath);
 }
 
 TEST(ControlLoadHandler, testLoadStoreLoadDefault) {
-    testLoadStoreLoadHelper(GET_TESTFILE("packaged_xopp/suite.xopp"), /*tol=*/1e-8);
+    testLoadStoreLoadHelper(GET_TESTFILE(u8"packaged_xopp/suite.xopp"), /*tol=*/1e-8);
 }
 
 // Backwards compatibility test that checks that full-precision float strings can be loaded.
 // See https://github.com/xournalpp/xournalpp/pull/4065
 TEST(ControlLoadHandler, testLoadStoreLoadFloatBwCompat) {
-    testLoadStoreLoadHelper(GET_TESTFILE("packaged_xopp/suite_float_bw_compat.xopp"), /*tol=*/1e-5);
+    testLoadStoreLoadHelper(GET_TESTFILE(u8"packaged_xopp/suite_float_bw_compat.xopp"), /*tol=*/1e-5);
 }
 
 TEST(ControlLoadHandler, testStrokeWidthRecovery) {
     LoadHandler handler;
-    Document* doc = handler.loadDocument(GET_TESTFILE("packaged_xopp/stroke/width_recovery.xopp"));
+    auto doc = handler.loadDocument(GET_TESTFILE(u8"packaged_xopp/stroke/width_recovery.xopp"));
 
     EXPECT_EQ((size_t)1, doc->getPageCount());
-    PageRef page = doc->getPage(0);
+    ConstPageRef page = doc->getPage(0);
 
     EXPECT_EQ((size_t)1, page->getLayerCount());
 
-    Layer* layer = (*(page->getLayers()))[0];
+    const Layer* layer = page->getLayersView()[0];
 
-    EXPECT_EQ(9U, layer->getElements().size());
+    EXPECT_EQ(9U, layer->getElementsView().size());
 
-    Stroke* s1 = (Stroke*)layer->getElements()[0];
+    auto* s1 = dynamic_cast<const Stroke*>(layer->getElementsView()[0]);
+    EXPECT_NE(s1, nullptr);
     EXPECT_EQ(ELEMENT_STROKE, s1->getType());
     for (auto& p: s1->getPointVector()) {
         EXPECT_EQ(p.z, Point::NO_PRESSURE);
     }
 
-    auto testPressureValues = [&elts = layer->getElements()](size_t n, const std::vector<double>& pressures) {
-        Stroke* s = (Stroke*)elts[n];
+    auto testPressureValues = [elts = layer->getElementsView()](size_t n, const std::vector<double>& pressures) {
+        auto* s = static_cast<const Stroke*>(elts[n]);
         printf("Testing stroke %zu\n", n);
         EXPECT_EQ(ELEMENT_STROKE, s->getType());
         EXPECT_EQ(Color(0x0000ff00), s->getColor());
@@ -526,29 +544,71 @@ TEST(ControlLoadHandler, testStrokeWidthRecovery) {
 
 TEST(ControlLoadHandler, testLoadStoreCJK) {
     LoadHandler handler;
-    auto filepath = string(GET_TESTFILE("cjk/测试.xopp"));
-    Document* doc = handler.loadDocument(fs::u8path(filepath));
-    ASSERT_NE(doc, nullptr);
+    auto filepath = std::u8string(GET_TESTFILE(u8"cjk/测试.xopp"));
+    auto doc = handler.loadDocument(fs::path(filepath));
+    ASSERT_NE(doc.get(), nullptr);
 
-    EXPECT_STREQ(doc->getPdfFilepath().filename().u8string().c_str(), u8"测试.pdf");
+    EXPECT_EQ(doc->getPdfFilepath().filename().u8string(), std::u8string_view{u8"测试.pdf"});
 
     EXPECT_EQ((size_t)2, doc->getPageCount());
-    const auto page = doc->getPage(0);
+    ConstPageRef page = doc->getPage(0);
 
     EXPECT_EQ((size_t)1, page->getLayerCount());
-    const auto* layer = (*page->getLayers())[0];
+    const auto* layer = page->getLayersView()[0];
 
-    const auto& elements = layer->getElements();
-    ASSERT_EQ((size_t)3, layer->getElements().size());
+    auto elements = layer->getElementsView();
+    ASSERT_EQ((size_t)3, layer->getElementsView().size());
 
-    auto check_element = [&](int i, const char* answer) {
+    auto check_element = [&](size_t i, const auto* answer) {
         EXPECT_EQ(ELEMENT_TEXT, elements[i]->getType());
-        auto* text = dynamic_cast<Text*>(elements[i]);
+        auto* text = dynamic_cast<const Text*>(elements[i]);
         ASSERT_NE(text, nullptr);
-        EXPECT_STREQ(text->getText().c_str(), answer);
+        EXPECT_STREQ(text->getText().c_str(), char_cast(answer));
     };
 
     check_element(0, u8"Test");
     check_element(1, u8"测试");
     check_element(2, u8"テスト");
 }
+
+TEST(ControlLoadHandler, testRelativePath) {
+    auto doc = LoadHandler().loadDocument(GET_TESTFILE(u8"load/relativePaths.xopp"));
+    ASSERT_TRUE(doc) << "Unable to load test file \"load/relativePaths.xopp\"";
+    const auto& pdffile = doc->getPdfFilepath();
+
+    auto check = [&pdffile](const fs::path& file, Util::PathStorageMode mode) {
+        const auto doc = LoadHandler().loadDocument(file);
+        ASSERT_TRUE(doc) << "Unable to load " << file.string();
+        EXPECT_TRUE(fs::equivalent(doc->getPdfFilepath().lexically_normal(), pdffile.lexically_normal()))
+                << "Paths \"" << doc->getPdfFilepath().string() << "\" and \"" << pdffile.string()
+                << "\" are not equivalent";
+        EXPECT_EQ(doc->getPathStorageMode(), mode);
+    };
+
+    auto saveReloadTest = [&](const fs::path& dir) {
+        std::cout << "Test saving in " << dir.string() << std::endl;
+        const fs::path outPath = dir / "xournalpp-test-units_ControlLoaderHandler_testRelativePath.xopp";
+        ASSERT_TRUE(!fs::exists(outPath));
+
+        SaveHandler saver;
+        saver.prepareSave(doc.get(), outPath);
+        saver.saveTo(outPath);
+        EXPECT_TRUE(saver.getErrorMessage().empty());
+
+        check(outPath, doc->getPathStorageMode());
+
+        fs::remove(outPath);
+    };
+
+    doc->setPathStorageMode(Util::PathStorageMode::AS_ABSOLUTE_PATH);
+    std::cout << "Mode PathStorageMode::AS_ABSOLUTE_PATH" << std::endl;
+
+    saveReloadTest(fs::temp_directory_path());
+    saveReloadTest(fs::current_path());
+
+    doc->setPathStorageMode(Util::PathStorageMode::AS_RELATIVE_PATH);
+    std::cout << "Mode PathStorageMode::AS_RELATIVE_PATH" << std::endl;
+
+    saveReloadTest(fs::temp_directory_path());
+    saveReloadTest(fs::current_path());
+}
